//400::ms => dur bpm;

// frequency modulation synth
public class Synth 
{
    // public variables
    int voices;
    Gain synthGain;
    JCRev synthReverb;
    LPF synthLPF;
    ADSR adsr;
    float detuneCents;
    
    // constructor
    5 => voices;
    SawOsc saw[voices];

    // Effects chain
    adsr => synthGain => synthLPF => synthReverb => dac;
    synthLPF.freq(20000);
    adsr.set(13::ms, 250::ms, 0.5, 15::ms);
    synthReverb.mix(.04);
    5.0 => detuneCents;
    
    // Connect oscillators, set gain
    for (0 => int i; i < voices; i++) {
        saw[i] => adsr;
        1.0 / voices => saw[i].gain;
    }

    fun void SynthNote(float duration, float freq)
    {
        // Create oscillator detuned voices, detuning of 5 cents
        Math.pow(2,(detuneCents/1200.0))*freq => float detunedFreqHigh;
        (detunedFreqHigh - freq) * 2 => float floatFreqRange; 
        detunedFreqHigh - floatFreqRange => float detunedFreqLow;

        floatFreqRange / voices => float freqStep;

        for (0 => int i; i < voices; i++) {
            detunedFreqLow + (freqStep * i) => saw[i].freq;
            //<<< i + " " + saw[i].freq() >>>;
        }

        adsr.keyOn();
        (duration / 2)::ms => now;
        adsr.keyOff();
        adsr.releaseTime() => now;
    }

    fun void reverb(float mix)
    {
        // Clamp mix between 0 and 0.5
        if (mix < 0.0) {
            0.0 => mix;
        } else if (mix > 0.5) {
            0.1 => mix;
        }
        mix => synthReverb.mix;
    }

    fun void gain(float gain)
    {
        // Clamp gain between 0 and 1
        if (gain < 0.0) {
            0.0 => gain;
        } else if (gain > 1.0) {
            1.0 => gain;
        }
        gain => synthGain.gain;
    }
    fun void lowpass(float freq)
    {
        // Clamp freq between 0 and 20000
        if (freq < 20.0) {
            0.0 => freq;
        } else if (freq > 20000.0) {
            20000.0 => freq;
        }
        freq => synthLPF.freq;
    }


}

/*
// Testing
Synth s;
440 => int i;

while (true)
{
    s.SynthNote(150, i);
    30 +=> i;
}
*/
